/*
	Slider Object
	Copyright © 2014,2017 Harry Whitfield

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public
	License along with this program; if not, write to the Free
	Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
	MA	02110-1301	USA

	Slider Object - version 0.6 - browser version
	21 November, 2017
	Copyright © 2014,2017 Harry Whitfield
	mailto:g6auc@arrl.net
*/

/*jslint browser, devel, multivar, this */

/*global newImage, moveObj, left, vtop, scale, window */

/*property
    clientX, clientY, getValue, hOff, hOffset, height, knob, knobDelta,
    knobSize, left, mouseIsDown, onmousedown, onmouseleave, onmousemove,
    onmouseup, ontouchend, ontouchmove, ontouchstart, preventDefault,
    prototype, scale, scrollX, scrollY, setValue, slideSize, sliderType, slot,
    stopPropagation, targetTouches, touchcancel, touchmove, vOff, vOffset,
    vtop, width
*/

var Slider = function (sliderType, hOffset, vOffset, width, height, left, vtop, imgFolder, zOrder, scale, report) {
	"use strict";
	/*
		sliderType must be one of:
			"horizontal"
			"vertical"
	*/

	var base = imgFolder + "/Sliders/",
		slotSrc = base + sliderType + "Slot.png",
		knobSrc = base + "sliderKnob.png",

		knobSize = 80,		// srcWidth and srcHeight of the knob image
		knobDelta = 40,		// knobSize / 2
		slideSize = 20,		// breadth of the slot image
		slideDelta = 30,	// knobDelta - slideSize / 2

		that = this;

	this.sliderType = sliderType;
	this.left = left;
	this.vtop = vtop;
	this.scale = scale;

	this.knobSize = knobSize;
	this.knobDelta = knobDelta;
	this.slideSize = slideSize;

	this.mouseIsDown = false;

	switch (sliderType) {
	case "horizontal":
		this.slot = newImage(hOffset, vOffset, width + knobSize - knobDelta, slideSize, slotSrc, zOrder, 1.0);
		this.slot.width = width + knobSize - knobDelta;
		this.slot.height = slideSize;
		break;
	case "vertical":
		this.slot = newImage(hOffset, vOffset, slideSize, height + knobSize - knobDelta, slotSrc, zOrder, 1.0);
		this.slot.width = slideSize;
		this.slot.height = height + knobSize - knobDelta;
		break;
	default:
		slotSrc = base + "errors.png";
		this.slot = newImage(hOffset, vOffset, 64, 64, slotSrc, zOrder, 1.0);
		this.slot.width = 64;
		this.slot.height = 64;
	}

	this.slot.hOffset = hOffset;
	this.slot.vOffset = vOffset;

	var noop = function (evt) {
		evt.preventDefault();
	};

	this.slot.onmousedown = noop;
	this.slot.onmouseup = noop;
	this.slot.onmouseleave = noop;
	this.slot.onmousemove = noop;

	this.slot.ontouchstart = noop;
	this.slot.ontouchend = noop;
	this.slot.touchcancel = noop;
	this.slot.touchmove = noop;

	switch (sliderType) {
	case "horizontal":
		this.knob = newImage(hOffset - knobDelta + slideSize, vOffset - slideDelta, knobSize, knobSize, knobSrc, zOrder + 1, 1.0);
		this.knob.hOffset = hOffset - knobDelta + slideSize;
		this.knob.vOffset = vOffset - slideDelta;
		this.knob.hOff = this.knob.hOffset;
		break;
	case "vertical":
		this.knob = newImage(hOffset - slideDelta, vOffset - knobDelta + slideSize, knobSize, knobSize, knobSrc, zOrder + 1, 1.0);
		this.knob.hOffset = hOffset - slideDelta;
		this.knob.vOffset = vOffset - knobDelta + slideSize;
		this.knob.vOff = this.knob.vOffset;
		break;
	}

	this.knob.onmousedown = function (evt) {
		evt.preventDefault();
		that.mouseIsDown = true;
	};

	this.knob.onmouseup = function (evt) {
		evt.preventDefault();
		that.mouseIsDown = false;
	};

	this.knob.onmouseleave = function (evt) {
		evt.preventDefault();
		that.mouseIsDown = false;
	};

    this.knob.ontouchstart = function (evt) {
        evt.stopPropagation();
        evt.preventDefault();
        that.mouseIsDown = true;
    };

	this.knob.ontouchend = function (evt) {
        evt.stopPropagation();
        evt.preventDefault();
        that.mouseIsDown = false;
    };

	this.knob.touchcancel = function (evt) {
        evt.stopPropagation();
        evt.preventDefault();
        that.mouseIsDown = false;
    };

	var knobMove = function (posX, posY) {	// posX, posY are scaled
        var hOff, vOff;
        var scrollX = window.scrollX;		// scrollX, scrollY are scaled
        var scrollY = window.scrollY;
        var lowX, lowY;
        var highX, highY;

		if (!that.mouseIsDown) {
			return;
		}

		switch (that.sliderType) {
		case "horizontal":
			hOff = posX + scrollX - that.left - knobDelta;
			lowX = that.slot.hOffset - knobDelta + slideSize;
			if (hOff < lowX) {
				hOff = lowX;
			}
			highX = lowX + that.slot.width - knobDelta;
			if (hOff > highX) {
				hOff = highX;
			}
			moveObj(that.knob, hOff, that.knob.vOffset);
			that.knob.hOff = hOff;
			report(hOff - lowX);
			break;
		case "vertical":
			vOff = posY + scrollY - that.vtop - knobDelta;
			lowY = that.slot.vOffset - knobDelta + slideSize;
			if (vOff < lowY) {
				vOff = lowY;
			}
			highY = lowY + that.slot.height - knobDelta;
			if (vOff > highY) {
				vOff = highY;
			}
			moveObj(that.knob, that.knob.hOffset, vOff);
			that.knob.vOff = vOff;
			report(vOff - lowY);
			break;
		default:
			alert("Invalid slider type: " + that.sliderType);
		}
	};

	this.knob.onmousemove = function (evt) {
        var posX, posY;

        evt.stopPropagation();
        evt.preventDefault();

      	posX = evt.clientX;
        posY = evt.clientY;
        knobMove(posX, posY);
    };

    this.knob.ontouchmove = function (evt) {
        var posX, posY;

        evt.stopPropagation();
        evt.preventDefault();

      	posX = evt.targetTouches[0].clientX;
        posY = evt.targetTouches[0].clientY;
        knobMove(posX, posY);
    };
};

Slider.prototype.getValue = function () {
	"use strict";
	var lowX, lowY;

	switch (this.sliderType) {
	case "horizontal":
		lowX = this.slot.hOffset - this.knobDelta + this.slideSize;
		return this.knob.hOff - lowX;
	case "vertical":
		lowY = this.slot.vOffset - this.knobDelta + this.slideSize;
		return this.knob.vOff - lowY;
	}
};

Slider.prototype.setValue = function (value) {
	"use strict";
    var hOff, vOff;
    var lowX, lowY;
    var highX, highY;

	switch (this.sliderType) {
	case "horizontal":
		lowX = this.slot.hOffset - this.knobDelta + this.slideSize;
		hOff = lowX + value;
		if (hOff < lowX) {
			hOff = lowX;
		}
		highX = lowX + this.slot.width - this.knobDelta;
		if (hOff > highX) {
			hOff = highX;
		}
		moveObj(this.knob, hOff, this.knob.vOffset);
		this.knob.hOff = hOff;
		return hOff - lowX;
	case "vertical":
		lowY = this.slot.vOffset - this.knobDelta + this.slideSize;
		vOff = lowY + value;
		if (vOff < lowY) {
			vOff = lowY;
		}
		highY = lowY + this.slot.height - this.knobDelta;
		if (vOff > highY) {
			vOff = highY;
		}
		moveObj(this.knob, this.knob.hOffset, vOff);
		this.knob.vOff = vOff;
		return vOff - lowY;
	}
};

/*
var testInfo = newDiv(0, 500, 1, "testInfo");

function print(value) {
	"use strict";
	testInfo.innerHTML = String(value);
}

var slider1 = new Slider("horizontal", 100, 100, 200, 20, left, vtop, "Resources/", 10, scale, print);
var slider2 = new Slider("vertical", 100, 200, 20, 200, left, vtop, "Resources/", 10, scale, print);
//alert("setValue: " + slider1.setValue(21));
//alert("setValue: " + slider2.setValue(157));
//alert("getValue: " + slider1.getValue());
//alert("getValue: " + slider2.getValue());
var slider3 = new Slider("vertical", 300, 200, 20, 200, left, vtop, "Resources/", 10, scale, print);
*/