//////////////////////////////////////////////////////////////////////////////////////////
//
// utmToLL - computes Latitude and Longitude given UTM Northing, UTM Easting and UTM Zone.
//
// Based on code posted by TreyA on Jul 18 2011 at 14:00 on
// http://stackoverflow.com/questions/343865/how-to-convert-from-utm-to-latlng-in-python-or-javascript
//
// Description:
//	This function converts UTM northing and UTM easting coordinates in a specified
//	UTM zone to the corresponding Latitude and Longitude values.
//
// Parameters:
//	northing	- (i) Northing (meters)
// 	easting		- (i) Easting  (meters)
//	utmZone		- (i) UTM Zone (number)
//  datum		- (i) datum (string)	"NAD27", "GRS80", "WGS84"
//	hemisphere	- (i) Hemisphere ("N" or "S")
//	lat			- (o) Latitude in degrees
//	lon			- (o) Longitude in degrees
//

/*jslint browser, this */

/*properties
    PI, cos, eccSq, lat, lon, pow, semiMajor, sin, sqrt, tan
*/

function utmToLL(northing, easting, utmZone, hemisphere, datum) {
	"use strict";
	// A geoid is the surface of an ellipsoid formed by rotating an ellipse
	// with semi-major axis a and semi-minor axis b about its minor (North-South) axis.
	//
	// The eccentricity e is defined as sqrt(1 - b*b/(a*a)).
	// The flattening is defined as 1 - b/a, and the inverse flattening as 1/f.
	// A datum is often defined using a and 1/f.
	// It is easily shown that e^2 = e*e = 1 - (1 - f)^2 = 2*f - f*f.
	// Below we define the geoids using a and e^2.
	//
	var LatLon = function (inLat, inLon) {
		this.lat = inLat;     // Store Latitude in decimal degrees
		this.lon = inLon;     // Store Longitude in decimal degrees
	};
	var degToRad = function (value) {
		return value * Math.PI / 180;
	};

	var radToDeg = function (value) {
		return value * 180 / Math.PI;
	};

	var geoid = function (datum) {
		switch (datum) {
		case "NAD27":
			return {semiMajor: 6378206.4, eccSq: 0.00676865799729};
		case "GRS80":
			return {semiMajor: 6378137.0, eccSq: 0.00669438002290078694};
		case "WGS84":
			return {semiMajor: 6378137.0, eccSq: 0.00669437999014131646};
		default:
			return {semiMajor: 6378137.0, eccSq: 0.00669437999014131646};	// WGS84
		}
	};

	// This is the lambda knot value in the reference
	var lngOrigin = degToRad(utmZone * 6 - 183);

	// The following set of class constants define characteristics of the
	// ellipsoid. These values need to be changed if a different datum is used.

	var falseNorth = (
		hemisphere === "N"
		? 0
		: 10000000
	);

	var eccSq = geoid(datum).eccSq;
	var ecc2Sq = eccSq / (1 - eccSq);

	var e1 = (1 - Math.sqrt(1 - eccSq)) / (1 + Math.sqrt(1 - eccSq));
	var e12 = e1 * e1;
	var e13 = e12 * e1;
	var e14 = e13 * e1;

	var semiMajor = geoid(datum).semiMajor;	// ellipsoidal semi-major axis (meters)
	var falseEast = 500000;          		// UTM East bias (meters)
	var scaleFactor = 0.9996;          		// scale at natural origin

  	// Calculate the Cassini projection parameters

	var m1 = (northing - falseNorth) / scaleFactor;
	var mu1 = m1 / (semiMajor * (
		1 - eccSq / 4 - 3 * eccSq * eccSq / 64 - 5 * eccSq * eccSq * eccSq / 256
	));

	var phi1 = (
		mu1 + (3 * e1 / 2 - 27 * e13 / 32) * Math.sin(2 * mu1) +
		(21 * e12 / 16 - 55 * e14 / 32) * Math.sin(4 * mu1) +
		(151 * e13 / 96) * Math.sin(6 * mu1) +
		(1097 * e14 / 512) * Math.sin(8 * mu1)
	);

	var sin2phi1 = Math.sin(phi1) * Math.sin(phi1);
	var rho1 = (semiMajor * (1 - eccSq)) / Math.pow(1 - eccSq * sin2phi1, 1.5);
	var nu1 = semiMajor / Math.sqrt(1 - eccSq * sin2phi1);

  	// Compute parameters as defined in the POSC specification.  t, c an d

	var t1 = Math.tan(phi1) * Math.tan(phi1);
	var t12 = t1 * t1;
	var c1 = ecc2Sq * Math.cos(phi1) * Math.cos(phi1);
	var c12 = c1 * c1;
	var d = (easting - falseEast) / (scaleFactor * nu1);
	var d2 = d * d;
	var d3 = d2 * d;
	var d4 = d3 * d;
	var d5 = d4 * d;
	var d6 = d5 * d;

  	// Compute the Latitude and Longitude and convert to degrees
	var lat = phi1 - nu1 * Math.tan(phi1) / rho1 * (
		d2 / 2 - (5 + 3 * t1 + 10 * c1 - 4 * c12 - 9 * ecc2Sq) * d4 / 24 +
		(61 + 90 * t1 + 298 * c1 + 45 * t12 - 252 * ecc2Sq - 3 * c12) * d6 / 720
	);
	var lon;

	lat = radToDeg(lat);

	lon = lngOrigin + (
		d - (1 + 2 * t1 + c1) * d3 / 6 +
		(5 - 2 * c1 + 28 * t1 - 3 * c12 + 8 * ecc2Sq + 24 * t12) * d5 / 120
	) / Math.cos(phi1);

	lon = radToDeg(lon);

	// Returns a LatLon object
	return new LatLon(lat, lon);
}

/*
// Test Examples

var datum = "WGS84";
var res;

res = utmToLL(5945970.05, 569606.36, 30, "N", datum);	// Rishworth Palace		WGS84	NAD27
print(res.lat.toFixed(8), res.lon.toFixed(8));			// 53.65784723 -1.94671111		53.65979566 -1.94669949

res = utmToLL(5411949, 448231, 31, "N", datum);			// Eiffel Tower			WGS84	NAD27
print(res.lat.toFixed(8), res.lon.toFixed(8));			// 48.85834509 2.29421445		48.86031514 2.29420935

res = utmToLL(6252309, 334897, 56, "S");				// Sydney Opera House	WGS84	NAD27
print(res.lat.toFixed(8), res.lon.toFixed(8));			// -33.85661692 151.21526523	-33.85661692 151.21526523

res = utmToLL(6170000, 373315, 21, "S");				// Buenos Aires Obelisk	WGS84	NAD27
print(res.lat.toFixed(8), res.lon.toFixed(8));			// -34.60402579 -58.38163268	-34.60402579 -58.38163268

res = utmToLL(4504682, 580720, 18, "N", datum);			// Statue of Liberty	WGS84	NAD27
print(res.lat.toFixed(8), res.lon.toFixed(8));			// 40.68908297 -74.04468950		40.69098678 -74.04468783

res = utmToLL(4883734, 643986, 10, "N", datum);			// Bend Airport			WGS84	NAD27
print(res.lat.toFixed(8), res.lon.toFixed(8));			// 44.09262978 -121.20129264	44.09457663 -121.20128564
*/